# @push.rocks/smartrequest
A module providing a drop-in replacement for the deprecated Request library, focusing on modern HTTP/HTTPS requests with support for form data, file uploads, JSON, binary data, and streams.

## Install
To install `@push.rocks/smartrequest`, use the following npm command:

```bash
npm install @push.rocks/smartrequest --save
```

This command will add `@push.rocks/smartrequest` to your project's dependencies.

## Usage
`@push.rocks/smartrequest` is designed as a versatile, modern HTTP client library for making HTTP/HTTPS requests. It supports a range of features, including handling form data, file uploads, JSON requests, binary data, streaming, and much more, all within a modern, promise-based API.

Below we will cover key usage scenarios of `@push.rocks/smartrequest`, showcasing its capabilities and providing you with a solid starting point to integrate it into your projects.

### Simple GET Request

For fetching data from a REST API or any web service that returns JSON:

```typescript
import { getJson } from '@push.rocks/smartrequest';

async function fetchGitHubUserInfo(username: string) {
  const response = await getJson(`https://api.github.com/users/${username}`);
  console.log(response.body); // The body contains the JSON response
}

fetchGitHubUserInfo('octocat');
```

The `getJson` function simplifies the process of sending a GET request and parsing the JSON response.

### POST Requests with JSON

When you need to send JSON data to a server, for example, creating a new resource:

```typescript
import { postJson } from '@push.rocks/smartrequest';

async function createTodoItem(todoDetails: { title: string; completed: boolean }) {
  const response = await postJson('https://jsonplaceholder.typicode.com/todos', {
    requestBody: todoDetails
  });
  console.log(response.body); // Log the created todo item
}

createTodoItem({ title: 'Implement smartrequest', completed: false });
```

`postJson` handles setting the appropriate content-type header and stringifies the JSON body.

### Handling Form Data and File Uploads

`@push.rocks/smartrequest` simplifies the process of uploading files and submitting form data to a server:

```typescript
import { postFormData, IFormField } from '@push.rocks/smartrequest';

async function uploadProfilePicture(formDataFields: IFormField[]) {
  await postFormData('https://api.example.com/upload', {}, formDataFields);
}

uploadProfilePicture([
  { name: 'avatar', type: 'filePath', payload: './path/to/avatar.jpg', fileName: 'avatar.jpg', contentType: 'image/jpeg' },
  { name: 'user_id', type: 'string', payload: '12345' }
]);
```

### Streaming Support

For cases when dealing with large datasets or streaming APIs, `@push.rocks/smartrequest` provides streaming capabilities:

```typescript
import { getStream } from '@push.rocks/smartrequest';

async function streamLargeFile(url: string) {
  const stream = await getStream(url);

  stream.on('data', (chunk) => {
    console.log('Received chunk of data.');
  });

  stream.on('end', () => {
    console.log('Stream ended.');
  });
}

streamLargeFile('https://example.com/largefile');
```

`getStream` allows you to handle data as it's received, which can be beneficial for performance and scalability.

### Advanced Options and Customization

`@push.rocks/smartrequest` is built to be flexible, allowing you to specify additional options to tailor requests to your needs:

```typescript
import { request, ISmartRequestOptions } from '@push.rocks/smartrequest';

async function customRequestExample() {
  const options: ISmartRequestOptions = {
    method: 'GET',
    headers: {
      'Custom-Header': 'Value'
    },
    keepAlive: true // Enables connection keep-alive
  };

  const response = await request('https://example.com/data', options);
  console.log(response.body);
}

customRequestExample();
```

`request` is the underlying function that powers the simpler `getJson`, `postJson`, etc., and provides you with full control over the HTTP request.

Through its comprehensive set of features tailored for modern web development, `@push.rocks/smartrequest` aims to provide developers with a powerful tool for handling HTTP/HTTPS requests efficiently. Whether it's a simple API call, handling form data, or processing streams, `@push.rocks/smartrequest` delivers a robust, type-safe solution to fit your project's requirements.

## License and Legal Information

This repository contains open-source code that is licensed under the MIT License. A copy of the MIT License can be found in the [license](license) file within this repository. 

**Please note:** The MIT License does not grant permission to use the trade names, trademarks, service marks, or product names of the project, except as required for reasonable and customary use in describing the origin of the work and reproducing the content of the NOTICE file.

### Trademarks

This project is owned and maintained by Task Venture Capital GmbH. The names and logos associated with Task Venture Capital GmbH and any related products or services are trademarks of Task Venture Capital GmbH and are not included within the scope of the MIT license granted herein. Use of these trademarks must comply with Task Venture Capital GmbH's Trademark Guidelines, and any usage must be approved in writing by Task Venture Capital GmbH.

### Company Information

Task Venture Capital GmbH  
Registered at District court Bremen HRB 35230 HB, Germany

For any legal inquiries or if you require further information, please contact us via email at hello@task.vc.

By using this repository, you acknowledge that you have read this section, agree to comply with its terms, and understand that the licensing of the code does not imply endorsement by Task Venture Capital GmbH of any derivative works.
